'use strict';

var ComponentRenderMixin = require('./component-render-mixin');
var StaticsMixin = require('./statics-mixin');

var debug = false;

module.exports = function (React) {
  return {
    enableDebugging: function enableDebugging(enable) {
      return debug = !!enable;
    },

    // @param additionalMixins? {Array|Object}
    // @param renderFn {Function}
    // @returns Component
    component: function component(additionalMixins, renderFn) {
      renderFn = renderFn || additionalMixins;
      additionalMixins = additionalMixins instanceof Function ? [] : [].concat(additionalMixins);

      var mixins = [ComponentRenderMixin, StaticsMixin].concat(additionalMixins);

      var render = function render() {
        if (debug) console.log('render <' + renderFn.name + ' />');
        return renderFn.call(this, this.props, this.props.statics, this.props.children);
      };

      var displayName = renderFn.name;

      return React.createClass({ displayName: displayName, mixins: mixins, render: render });
    }
  };
};